@@ -21,20 +21,20 @@ class UtilTest(TestCase):
 
     def test_lmhash(self):
         for secret, hash in [
-            ("OLDPASSWORD", "c9b81d939d6fd80cd408e6b105741864"),
-            ("NEWPASSWORD", '09eeab5aa415d6e4d408e6b105741864'),
-            ("welcome", "c23413a8a1e7665faad3b435b51404ee"),
+            ("OLDPASSWORD", u"c9b81d939d6fd80cd408e6b105741864"),
+            ("NEWPASSWORD", u'09eeab5aa415d6e4d408e6b105741864'),
+            ("welcome", u"c23413a8a1e7665faad3b435b51404ee"),
             ]:
             result = mod.raw_lmhash(secret, hex=True)
-            self.assertEquals(result, hash)
+            self.assertEqual(result, hash)
 
     def test_nthash(self):
         for secret, hash in [
-            ("OLDPASSWORD", "6677b2c394311355b54f25eec5bfacf5"),
-            ("NEWPASSWORD", "256781a62031289d3c2c98c14f1efc8c"),
+            ("OLDPASSWORD", u"6677b2c394311355b54f25eec5bfacf5"),
+            ("NEWPASSWORD", u"256781a62031289d3c2c98c14f1efc8c"),
             ]:
             result = mod.raw_nthash(secret, hex=True)
-            self.assertEquals(result, hash)
+            self.assertEqual(result, hash)
 
 #=========================================================
 #EOF
